"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_postmortems_postmortemlistresult import (
    V3IncidentsPostmortemsPostmortemListResult,
    V3IncidentsPostmortemsPostmortemListResultTypedDict,
)
from .v3_incidents_postmortems_totalpostmortemcount import (
    V3IncidentsPostmortemsTotalPostmortemCount,
    V3IncidentsPostmortemsTotalPostmortemCountTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import List
from typing_extensions import Annotated, TypedDict


class PostmortemsGetAllPostmortemsRequestTypedDict(TypedDict):
    from_date: str
    r"""Provide date in RFC3339 format"""
    to_date: str
    r"""Provide date in RFC3339 format"""
    owner_id: str
    r"""Here owner_id represents team_id"""
    limit: int


class PostmortemsGetAllPostmortemsRequest(BaseModel):
    from_date: Annotated[
        str,
        pydantic.Field(alias="fromDate"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]
    r"""Provide date in RFC3339 format"""

    to_date: Annotated[
        str,
        pydantic.Field(alias="toDate"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]
    r"""Provide date in RFC3339 format"""

    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Here owner_id represents team_id"""

    limit: Annotated[
        int, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]


class PostmortemsGetAllPostmortemsDataTypedDict(TypedDict):
    result: List[V3IncidentsPostmortemsPostmortemListResultTypedDict]
    total_count: List[V3IncidentsPostmortemsTotalPostmortemCountTypedDict]


class PostmortemsGetAllPostmortemsData(BaseModel):
    result: List[V3IncidentsPostmortemsPostmortemListResult]

    total_count: List[V3IncidentsPostmortemsTotalPostmortemCount]


class PostmortemsGetAllPostmortemsResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[PostmortemsGetAllPostmortemsDataTypedDict]


class PostmortemsGetAllPostmortemsResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[PostmortemsGetAllPostmortemsData]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V3IncidentsTagsAnalyticsMetricsTypedDict(TypedDict):
    r"""Analytics metrics for TTA (Time to Acknowledge) and TTR (Time to Resolve)."""

    time: int
    user_id: str
    escalation_policy_id: str
    squad_id: str


class V3IncidentsTagsAnalyticsMetrics(BaseModel):
    r"""Analytics metrics for TTA (Time to Acknowledge) and TTR (Time to Resolve)."""

    time: int

    user_id: Annotated[str, pydantic.Field(alias="userId")]

    escalation_policy_id: Annotated[str, pydantic.Field(alias="escalationPolicyId")]

    squad_id: Annotated[str, pydantic.Field(alias="squadId")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional


class SquadsMembers(BaseSDK):
    def update(
        self,
        *,
        squad_id: str,
        member_id: str,
        replace_with: str,
        role: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsUpdateSquadMemberResponse:
        r"""Update Squad Member

        This endpoint is used to update a squad member's role and is only accessible if your organization is using the OBAC permission model.

        If you're attempting to update a member's role from 'owner' to 'member', and that 'owner' is the last squad owner, then the 'replaceWith' query parameter is required. Setting 'replaceWith' to 'member' will promote the member to the role of owner.

        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-create` scope.

        :param squad_id:
        :param member_id:
        :param replace_with:
        :param role:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsUpdateSquadMemberRequest(
            squad_id=squad_id,
            member_id=member_id,
            replace_with=replace_with,
            v4_squads_update_squad_member_request=models.V4SquadsUpdateSquadMemberRequest(
                role=role,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v4/squads/{squadID}/members/{memberID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_squads_update_squad_member_request,
                False,
                False,
                "json",
                models.V4SquadsUpdateSquadMemberRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_updateSquadMember",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.V4SquadsUpdateSquadMemberResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        squad_id: str,
        member_id: str,
        replace_with: str,
        role: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4SquadsUpdateSquadMemberResponse:
        r"""Update Squad Member

        This endpoint is used to update a squad member's role and is only accessible if your organization is using the OBAC permission model.

        If you're attempting to update a member's role from 'owner' to 'member', and that 'owner' is the last squad owner, then the 'replaceWith' query parameter is required. Setting 'replaceWith' to 'member' will promote the member to the role of owner.

        Requires `access_token` as a `Bearer {{token}}` in the `Authorization` header with `squad-create` scope.

        :param squad_id:
        :param member_id:
        :param replace_with:
        :param role:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SquadsUpdateSquadMemberRequest(
            squad_id=squad_id,
            member_id=member_id,
            replace_with=replace_with,
            v4_squads_update_squad_member_request=models.V4SquadsUpdateSquadMemberRequest(
                role=role,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v4/squads/{squadID}/members/{memberID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_squads_update_squad_member_request,
                False,
                False,
                "json",
                models.V4SquadsUpdateSquadMemberRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Squads_updateSquadMember",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.V4SquadsUpdateSquadMemberResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_extensions_webhooks_webhookfilter import (
    V3ExtensionsWebhooksWebhookFilter,
    V3ExtensionsWebhooksWebhookFilterTypedDict,
)
from .v3_extensions_webhooks_webhooktrigger import (
    V3ExtensionsWebhooksWebhookTrigger,
    V3ExtensionsWebhooksWebhookTriggerTypedDict,
)
from .v3_extensions_webhooks_webhookurl import (
    V3ExtensionsWebhooksWebhookURL,
    V3ExtensionsWebhooksWebhookURLTypedDict,
)
from pydantic import model_serializer
from squadcast_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ExtensionsWebhooksWebhookResponseHeaderTypedDict(TypedDict):
    pass


class V3ExtensionsWebhooksWebhookResponseHeader(BaseModel):
    pass


class V3ExtensionsWebhooksWebhookResponseTypedDict(TypedDict):
    name: str
    r"""Name of the webhook"""
    triggers: List[V3ExtensionsWebhooksWebhookTriggerTypedDict]
    r"""A list of triggers for this webhook"""
    urls: List[V3ExtensionsWebhooksWebhookURLTypedDict]
    r"""A list of URLs to which the webhook payload will be sent"""
    trigger_type: str
    r"""Type of trigger"""
    id: str
    created_at: str
    updated_at: str
    author_id: str
    slug: str
    description: NotRequired[str]
    r"""Description of the webhook"""
    header: NotRequired[Nullable[V3ExtensionsWebhooksWebhookResponseHeaderTypedDict]]
    r"""Headers to be sent with the webhook"""
    filters: NotRequired[Nullable[V3ExtensionsWebhooksWebhookFilterTypedDict]]
    r"""Filters to apply to the webhook"""
    max_retry: NotRequired[int]
    r"""Maximum number of retries for the webhook"""
    teams: NotRequired[Nullable[List[str]]]
    r"""List of team IDs to which this webhook is applicable"""
    is_all_teams_configured: NotRequired[bool]
    r"""Set to true if the webhook is configured for all teams"""
    custom_payload_template_slug: NotRequired[str]
    r"""Slug of the custom payload template"""
    language: NotRequired[str]
    r"""Language for the webhook payload"""
    mail_ids: NotRequired[Nullable[List[str]]]
    r"""List of email IDs for notification"""
    custom_payload: NotRequired[str]
    r"""Custom payload for the webhook"""
    payload_type: NotRequired[str]
    r"""Type of payload"""
    organization_id: NotRequired[str]
    version: NotRequired[str]


class V3ExtensionsWebhooksWebhookResponse(BaseModel):
    name: str
    r"""Name of the webhook"""

    triggers: List[V3ExtensionsWebhooksWebhookTrigger]
    r"""A list of triggers for this webhook"""

    urls: List[V3ExtensionsWebhooksWebhookURL]
    r"""A list of URLs to which the webhook payload will be sent"""

    trigger_type: str
    r"""Type of trigger"""

    id: str

    created_at: str

    updated_at: str

    author_id: str

    slug: str

    description: Optional[str] = None
    r"""Description of the webhook"""

    header: OptionalNullable[V3ExtensionsWebhooksWebhookResponseHeader] = UNSET
    r"""Headers to be sent with the webhook"""

    filters: OptionalNullable[V3ExtensionsWebhooksWebhookFilter] = UNSET
    r"""Filters to apply to the webhook"""

    max_retry: Optional[int] = None
    r"""Maximum number of retries for the webhook"""

    teams: OptionalNullable[List[str]] = UNSET
    r"""List of team IDs to which this webhook is applicable"""

    is_all_teams_configured: Optional[bool] = None
    r"""Set to true if the webhook is configured for all teams"""

    custom_payload_template_slug: Optional[str] = None
    r"""Slug of the custom payload template"""

    language: Optional[str] = None
    r"""Language for the webhook payload"""

    mail_ids: OptionalNullable[List[str]] = UNSET
    r"""List of email IDs for notification"""

    custom_payload: Optional[str] = None
    r"""Custom payload for the webhook"""

    payload_type: Optional[str] = None
    r"""Type of payload"""

    organization_id: Optional[str] = None

    version: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "header",
            "filters",
            "max_retry",
            "teams",
            "is_all_teams_configured",
            "custom_payload_template_slug",
            "language",
            "mail_ids",
            "custom_payload",
            "payload_type",
            "organization_id",
            "version",
        ]
        nullable_fields = ["header", "filters", "teams", "mail_ids"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_actionresponse import (
    V3WorkflowsActionResponse,
    V3WorkflowsActionResponseTypedDict,
)
from .v3_workflows_tag import V3WorkflowsTag, V3WorkflowsTagTypedDict
from .v3_workflows_workflowfilters import (
    V3WorkflowsWorkflowFilters,
    V3WorkflowsWorkflowFiltersTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3WorkflowsUpdateWorkflowAPIResponseTypedDict(TypedDict):
    id: NotRequired[int]
    owner_id: NotRequired[str]
    title: NotRequired[str]
    description: NotRequired[str]
    enabled: NotRequired[bool]
    trigger: NotRequired[str]
    filters: NotRequired[V3WorkflowsWorkflowFiltersTypedDict]
    tags: NotRequired[List[V3WorkflowsTagTypedDict]]
    actions: NotRequired[List[V3WorkflowsActionResponseTypedDict]]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class V3WorkflowsUpdateWorkflowAPIResponse(BaseModel):
    id: Optional[int] = None

    owner_id: Optional[str] = None

    title: Optional[str] = None

    description: Optional[str] = None

    enabled: Optional[bool] = None

    trigger: Optional[str] = None

    filters: Optional[V3WorkflowsWorkflowFilters] = None

    tags: Optional[List[V3WorkflowsTag]] = None

    actions: Optional[List[V3WorkflowsActionResponse]] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_users_apitokenresponse import (
    V3UsersAPITokenResponse,
    V3UsersAPITokenResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class UsersCreateTokenRequestTypedDict(TypedDict):
    user_id: str


class UsersCreateTokenRequest(BaseModel):
    user_id: str


class UsersCreateTokenResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3UsersAPITokenResponseTypedDict


class UsersCreateTokenResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3UsersAPITokenResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_users_globalrbacabilityname import V3UsersGlobalRBACAbilityName
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3UsersUpdateUserAbilitiesRequestDataTypedDict(TypedDict):
    user_id: str
    abilities: List[V3UsersGlobalRBACAbilityName]


class V3UsersUpdateUserAbilitiesRequestData(BaseModel):
    user_id: str

    abilities: List[V3UsersGlobalRBACAbilityName]


class V3UsersUpdateUserAbilitiesRequestTypedDict(TypedDict):
    data: List[V3UsersUpdateUserAbilitiesRequestDataTypedDict]


class V3UsersUpdateUserAbilitiesRequest(BaseModel):
    data: List[V3UsersUpdateUserAbilitiesRequestData]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V4ParticipantTypedDict(TypedDict):
    id: str
    type: str


class V4Participant(BaseModel):
    id: str

    type: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSqTriggerManualWebhookUpdateName = Literal["sq_trigger_manual_webhook",]


class V3WorkflowsSqTriggerManualWebhookUpdateDataTypedDict(TypedDict):
    id: NotRequired[str]


class V3WorkflowsSqTriggerManualWebhookUpdateData(BaseModel):
    id: Optional[str] = None


class V3WorkflowsSqTriggerManualWebhookUpdateTypedDict(TypedDict):
    name: V3WorkflowsSqTriggerManualWebhookUpdateName
    data: NotRequired[V3WorkflowsSqTriggerManualWebhookUpdateDataTypedDict]


class V3WorkflowsSqTriggerManualWebhookUpdate(BaseModel):
    name: V3WorkflowsSqTriggerManualWebhookUpdateName

    data: Optional[V3WorkflowsSqTriggerManualWebhookUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsPostmortemsPostmortemFollowUpTypedDict(TypedDict):
    r"""Represents a single follow-up task in a postmortem."""

    task: str
    is_done: bool


class V3IncidentsPostmortemsPostmortemFollowUp(BaseModel):
    r"""Represents a single follow-up task in a postmortem."""

    task: str

    is_done: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_teams_teamresponse import V3TeamsTeamResponse, V3TeamsTeamResponseTypedDict
from .v3_teams_updateteamrequest import (
    V3TeamsUpdateTeamRequest,
    V3TeamsUpdateTeamRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class TeamsUpdateTeamRequestTypedDict(TypedDict):
    team_id: str
    v3_teams_update_team_request: V3TeamsUpdateTeamRequestTypedDict


class TeamsUpdateTeamRequest(BaseModel):
    team_id: Annotated[
        str,
        pydantic.Field(alias="teamId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v3_teams_update_team_request: Annotated[
        V3TeamsUpdateTeamRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class TeamsUpdateTeamResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3TeamsTeamResponseTypedDict


class TeamsUpdateTeamResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3TeamsTeamResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3UsersUpdateUserRequestRole = Literal[
    "account_owner",
    "user",
    "stakeholder",
]


class V3UsersUpdateUserRequestTypedDict(TypedDict):
    role: V3UsersUpdateUserRequestRole


class V3UsersUpdateUserRequest(BaseModel):
    role: V3UsersUpdateUserRequestRole



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V3AuditLogsTeamTypedDict(TypedDict):
    r"""Represents a team in audit logs"""

    id: str
    name: str
    is_deleted: bool


class V3AuditLogsTeam(BaseModel):
    r"""Represents a team in audit logs"""

    id: str

    name: str

    is_deleted: Annotated[bool, pydantic.Field(alias="isDeleted")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_overrideparticipantgroup import (
    V4OverrideParticipantGroup,
    V4OverrideParticipantGroupTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class V4OverrideResponseTypedDict(TypedDict):
    id: int
    org_id: str
    team_id: str
    schedule_id: int
    start_time: str
    end_time: str
    reason: str
    overridden_participant: V4OverrideParticipantGroupTypedDict
    override_with: V4OverrideParticipantGroupTypedDict
    created_at: str
    updated_at: str


class V4OverrideResponse(BaseModel):
    id: int

    org_id: Annotated[str, pydantic.Field(alias="orgID")]

    team_id: Annotated[str, pydantic.Field(alias="teamID")]

    schedule_id: Annotated[int, pydantic.Field(alias="scheduleID")]

    start_time: Annotated[str, pydantic.Field(alias="startTime")]

    end_time: Annotated[str, pydantic.Field(alias="endTime")]

    reason: str

    overridden_participant: Annotated[
        V4OverrideParticipantGroup, pydantic.Field(alias="overriddenParticipant")
    ]

    override_with: Annotated[
        V4OverrideParticipantGroup, pydantic.Field(alias="overrideWith")
    ]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3WorkflowsComponentAndImpactTypedDict(TypedDict):
    component_id: int
    impact_status_id: int


class V3WorkflowsComponentAndImpact(BaseModel):
    component_id: int

    impact_status_id: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_liststatuspagesresponse import (
    V4StatusPagesListStatusPagesResponse,
    V4StatusPagesListStatusPagesResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, Optional
from typing_extensions import Annotated, TypedDict


class StatusPagesListStatusPagesRequestTypedDict(TypedDict):
    page_size: int
    page_number: int
    filters_is_public: str
    team_id: str


class StatusPagesListStatusPagesRequest(BaseModel):
    page_size: Annotated[
        int,
        pydantic.Field(alias="pageSize"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    page_number: Annotated[
        int,
        pydantic.Field(alias="pageNumber"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    filters_is_public: Annotated[
        str,
        pydantic.Field(alias="filters.isPublic"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    team_id: Annotated[
        str,
        pydantic.Field(alias="teamID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]


class StatusPagesListStatusPagesResponseTypedDict(TypedDict):
    result: V4StatusPagesListStatusPagesResponseTypedDict


class StatusPagesListStatusPagesResponse(BaseModel):
    next: Callable[[], Optional[StatusPagesListStatusPagesResponse]]

    result: V4StatusPagesListStatusPagesResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_users_userresponse import V3UsersUserResponse, V3UsersUserResponseTypedDict
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class UsersGetAllUsersResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3UsersUserResponseTypedDict]


class UsersGetAllUsersResponse(BaseModel):
    r"""The request has succeeded."""

    data: List[V3UsersUserResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesJiraStatusMapTypedDict(TypedDict):
    jira_status: str
    system_status: str


class V3ServicesJiraStatusMap(BaseModel):
    jira_status: str

    system_status: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesJiraStatusTypedDict(TypedDict):
    id: str
    name: str


class V3ServicesJiraStatus(BaseModel):
    id: str

    name: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsMsTeamsMessageChannelName = Literal["msteams_message_channel",]


class V3WorkflowsMsTeamsMessageChannelDataTypedDict(TypedDict):
    channel_id: str
    message: str


class V3WorkflowsMsTeamsMessageChannelData(BaseModel):
    channel_id: str

    message: str


class V3WorkflowsMsTeamsMessageChannelTypedDict(TypedDict):
    name: V3WorkflowsMsTeamsMessageChannelName
    data: V3WorkflowsMsTeamsMessageChannelDataTypedDict


class V3WorkflowsMsTeamsMessageChannel(BaseModel):
    name: V3WorkflowsMsTeamsMessageChannelName

    data: V3WorkflowsMsTeamsMessageChannelData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class StepTypedDict(TypedDict):
    content: str
    completed: bool
    completed_at: NotRequired[datetime]


class Step(BaseModel):
    content: str

    completed: bool

    completed_at: Optional[datetime] = None


class V3IncidentsRunbooksRunbookResponseTypedDict(TypedDict):
    incident_id: str
    runbook_id: str
    name: str
    steps: List[StepTypedDict]
    id: NotRequired[str]
    deleted: NotRequired[bool]
    deleted_at: NotRequired[datetime]


class V3IncidentsRunbooksRunbookResponse(BaseModel):
    incident_id: str

    runbook_id: str

    name: str

    steps: List[Step]

    id: Optional[str] = None

    deleted: Optional[bool] = None

    deleted_at: Optional[datetime] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_notes_createnoterequest import (
    V3IncidentsNotesCreateNoteRequest,
    V3IncidentsNotesCreateNoteRequestTypedDict,
)
from .v3_incidents_notes_noteresponse import (
    V3IncidentsNotesNoteResponse,
    V3IncidentsNotesNoteResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class NotesCreateNotesRequestTypedDict(TypedDict):
    incident_id: str
    r"""Required"""
    v3_incidents_notes_create_note_request: V3IncidentsNotesCreateNoteRequestTypedDict


class NotesCreateNotesRequest(BaseModel):
    incident_id: Annotated[
        str,
        pydantic.Field(alias="IncidentId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Required"""

    v3_incidents_notes_create_note_request: Annotated[
        V3IncidentsNotesCreateNoteRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class NotesCreateNotesBodyTypedDict(TypedDict):
    r"""The body type of the operation request or response."""

    data: V3IncidentsNotesNoteResponseTypedDict


class NotesCreateNotesBody(BaseModel):
    r"""The body type of the operation request or response."""

    data: V3IncidentsNotesNoteResponse


class NotesCreateNotesResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    body: NotesCreateNotesBodyTypedDict
    r"""The body type of the operation request or response."""


class NotesCreateNotesResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    body: NotesCreateNotesBody
    r"""The body type of the operation request or response."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_globaleventruleresponse import (
    V3GlobalEventRulesGlobalEventRuleResponse,
    V3GlobalEventRulesGlobalEventRuleResponseTypedDict,
)
from .v3_globaleventrules_updateglobaleventrulerequest import (
    V3GlobalEventRulesUpdateGlobalEventRuleRequest,
    V3GlobalEventRulesUpdateGlobalEventRuleRequestTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class GlobalEventRulesUpdateGlobalEventRuleByIDRequestTypedDict(TypedDict):
    ger_id: int
    v3_global_event_rules_update_global_event_rule_request: (
        V3GlobalEventRulesUpdateGlobalEventRuleRequestTypedDict
    )


class GlobalEventRulesUpdateGlobalEventRuleByIDRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    v3_global_event_rules_update_global_event_rule_request: Annotated[
        V3GlobalEventRulesUpdateGlobalEventRuleRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class GlobalEventRulesUpdateGlobalEventRuleByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesGlobalEventRuleResponseTypedDict


class GlobalEventRulesUpdateGlobalEventRuleByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesGlobalEventRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3GlobalEventRulesEntityOwnerUpdateType = Literal[
    "team",
    "user",
    "squad",
]


class V3GlobalEventRulesEntityOwnerUpdateTypedDict(TypedDict):
    id: NotRequired[str]
    type: NotRequired[V3GlobalEventRulesEntityOwnerUpdateType]


class V3GlobalEventRulesEntityOwnerUpdate(BaseModel):
    id: Optional[str] = None

    type: Optional[V3GlobalEventRulesEntityOwnerUpdateType] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


Condition = Literal[
    "and",
    "or",
]


class V3WorkflowsWorkflowFiltersTypedDict(TypedDict):
    type: NotRequired[str]
    key: NotRequired[str]
    value: NotRequired[str]
    condition: NotRequired[Condition]
    filters: NotRequired[List[V3WorkflowsWorkflowFiltersTypedDict]]


class V3WorkflowsWorkflowFilters(BaseModel):
    type: Optional[str] = None

    key: Optional[str] = None

    value: Optional[str] = None

    condition: Optional[Condition] = None

    filters: Optional[List[V3WorkflowsWorkflowFilters]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_extensions_webhooks_webhookfiltercondition import (
    V3ExtensionsWebhooksWebhookFilterCondition,
    V3ExtensionsWebhooksWebhookFilterConditionTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3ExtensionsWebhooksWebhookFilterTypedDict(TypedDict):
    op: str
    conditions: List[V3ExtensionsWebhooksWebhookFilterConditionTypedDict]


class V3ExtensionsWebhooksWebhookFilter(BaseModel):
    op: str

    conditions: List[V3ExtensionsWebhooksWebhookFilterCondition]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_scheduleresponse import V4ScheduleResponse, V4ScheduleResponseTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class SchedulesGetScheduleByIDRequestTypedDict(TypedDict):
    schedule_id: str


class SchedulesGetScheduleByIDRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class SchedulesGetScheduleByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4ScheduleResponseTypedDict


class SchedulesGetScheduleByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4ScheduleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_snoozenotifications_reassignto import (
    V3IncidentsSnoozeNotificationsReassignTo,
    V3IncidentsSnoozeNotificationsReassignToTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsSnoozeNotificationsUnsnoozeIncidentResponseTypedDict(TypedDict):
    incident_id: str
    is_snoozed: bool
    reassign_to: V3IncidentsSnoozeNotificationsReassignToTypedDict


class V3IncidentsSnoozeNotificationsUnsnoozeIncidentResponse(BaseModel):
    incident_id: str

    is_snoozed: bool

    reassign_to: V3IncidentsSnoozeNotificationsReassignTo



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_services_taggingrules_expressionbranch import (
    V3ServicesTaggingRulesExpressionBranch,
    V3ServicesTaggingRulesExpressionBranchTypedDict,
)
from .v3_services_taggingrules_tagsobject import (
    V3ServicesTaggingRulesTagsObject,
    V3ServicesTaggingRulesTagsObjectTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ServicesTaggingRulesTaggingRuleResponseTypedDict(TypedDict):
    expression: NotRequired[str]
    tags: NotRequired[V3ServicesTaggingRulesTagsObjectTypedDict]
    r"""The 'tags' field cannot be empty unless the 'expression' or 'basic_expression' explicitly uses 'addTag' or 'addTags' functions."""
    is_basic: NotRequired[bool]
    basic_expression: NotRequired[List[V3ServicesTaggingRulesExpressionBranchTypedDict]]


class V3ServicesTaggingRulesTaggingRuleResponse(BaseModel):
    expression: Optional[str] = None

    tags: Optional[V3ServicesTaggingRulesTagsObject] = None
    r"""The 'tags' field cannot be empty unless the 'expression' or 'basic_expression' explicitly uses 'addTag' or 'addTags' functions."""

    is_basic: Optional[bool] = None

    basic_expression: Optional[List[V3ServicesTaggingRulesExpressionBranch]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


V3WorkflowsSqMarkIncidentSLOAffectingUpdateName = Literal[
    "sq_mark_incident_slo_affecting",
]


class V3WorkflowsSqMarkIncidentSLOAffectingUpdateDataTypedDict(TypedDict):
    slis: NotRequired[List[str]]
    slo: NotRequired[int]


class V3WorkflowsSqMarkIncidentSLOAffectingUpdateData(BaseModel):
    slis: Optional[List[str]] = None

    slo: Optional[int] = None


class V3WorkflowsSqMarkIncidentSLOAffectingUpdateTypedDict(TypedDict):
    name: V3WorkflowsSqMarkIncidentSLOAffectingUpdateName
    data: NotRequired[V3WorkflowsSqMarkIncidentSLOAffectingUpdateDataTypedDict]


class V3WorkflowsSqMarkIncidentSLOAffectingUpdate(BaseModel):
    name: V3WorkflowsSqMarkIncidentSLOAffectingUpdateName

    data: Optional[V3WorkflowsSqMarkIncidentSLOAffectingUpdateData] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesUpdateStatusPageByIDRequestThemeColorTypedDict(TypedDict):
    primary: str
    secondary: str


class V4StatusPagesUpdateStatusPageByIDRequestThemeColor(BaseModel):
    primary: str

    secondary: str


class V4StatusPagesUpdateStatusPageByIDRequestComponent1TypedDict(TypedDict):
    type: NotRequired[str]
    name: NotRequired[str]
    allow_subscription: NotRequired[bool]
    description: NotRequired[str]
    service_id: NotRequired[str]


class V4StatusPagesUpdateStatusPageByIDRequestComponent1(BaseModel):
    type: Optional[str] = None

    name: Optional[str] = None

    allow_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowSubscription")
    ] = None

    description: Optional[str] = None

    service_id: Annotated[Optional[str], pydantic.Field(alias="serviceID")] = None


class V4StatusPagesUpdateStatusPageByIDRequestComponent2TypedDict(TypedDict):
    type: NotRequired[str]
    name: NotRequired[str]
    allow_subscription: NotRequired[bool]
    description: NotRequired[str]
    components: NotRequired[
        List[V4StatusPagesUpdateStatusPageByIDRequestComponent1TypedDict]
    ]


class V4StatusPagesUpdateStatusPageByIDRequestComponent2(BaseModel):
    type: Optional[str] = None

    name: Optional[str] = None

    allow_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowSubscription")
    ] = None

    description: Optional[str] = None

    components: Optional[List[V4StatusPagesUpdateStatusPageByIDRequestComponent1]] = (
        None
    )


class V4StatusPagesUpdateStatusPageByIDRequestTypedDict(TypedDict):
    name: str
    is_public: bool
    domain_name: str
    team_id: str
    theme_color: V4StatusPagesUpdateStatusPageByIDRequestThemeColorTypedDict
    contact_email: str
    owner_type: str
    owner_id: str
    description: NotRequired[str]
    custom_domain_name: NotRequired[str]
    logo_url: NotRequired[str]
    timezone: NotRequired[str]
    allow_components_subscription: NotRequired[bool]
    allow_maintenance_subscription: NotRequired[bool]
    allow_webhook_subscription: NotRequired[bool]
    components: NotRequired[
        List[V4StatusPagesUpdateStatusPageByIDRequestComponent2TypedDict]
    ]
    is_custom_domain_enabled: NotRequired[bool]
    hide_from_search_engines: NotRequired[bool]


class V4StatusPagesUpdateStatusPageByIDRequest(BaseModel):
    name: str

    is_public: Annotated[bool, pydantic.Field(alias="isPublic")]

    domain_name: Annotated[str, pydantic.Field(alias="domainName")]

    team_id: Annotated[str, pydantic.Field(alias="teamID")]

    theme_color: Annotated[
        V4StatusPagesUpdateStatusPageByIDRequestThemeColor,
        pydantic.Field(alias="themeColor"),
    ]

    contact_email: Annotated[str, pydantic.Field(alias="contactEmail")]

    owner_type: Annotated[str, pydantic.Field(alias="ownerType")]

    owner_id: Annotated[str, pydantic.Field(alias="ownerID")]

    description: Optional[str] = None

    custom_domain_name: Annotated[
        Optional[str], pydantic.Field(alias="customDomainName")
    ] = None

    logo_url: Annotated[Optional[str], pydantic.Field(alias="logoUrl")] = None

    timezone: Optional[str] = None

    allow_components_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowComponentsSubscription")
    ] = None

    allow_maintenance_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowMaintenanceSubscription")
    ] = None

    allow_webhook_subscription: Annotated[
        Optional[bool], pydantic.Field(alias="allowWebhookSubscription")
    ] = None

    components: Optional[List[V4StatusPagesUpdateStatusPageByIDRequestComponent2]] = (
        None
    )

    is_custom_domain_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isCustomDomainEnabled")
    ] = None

    hide_from_search_engines: Annotated[
        Optional[bool], pydantic.Field(alias="hideFromSearchEngines")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4ShiftTimeSlotTypedDict(TypedDict):
    start_hour: int
    start_min: int
    duration: int
    day_of_week: NotRequired[str]


class V4ShiftTimeSlot(BaseModel):
    start_hour: Annotated[int, pydantic.Field(alias="startHour")]

    start_min: Annotated[int, pydantic.Field(alias="startMin")]

    duration: int

    day_of_week: Annotated[Optional[str], pydantic.Field(alias="dayOfWeek")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsCommunicationCardsCreateSlackChannelRequestTypedDict(TypedDict):
    r"""Request body for creating a Slack channel in a communication card."""

    channel_name: str
    incident_id: str


class V3IncidentsCommunicationCardsCreateSlackChannelRequest(BaseModel):
    r"""Request body for creating a Slack channel in a communication card."""

    channel_name: str

    incident_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_webforms_webformresponse import (
    V3WebformsWebformResponse,
    V3WebformsWebformResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, QueryParamMetadata
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebformsGetAllWebformsRequestTypedDict(TypedDict):
    owner_id: str
    page_number: NotRequired[int]
    page_size: NotRequired[int]


class WebformsGetAllWebformsRequest(BaseModel):
    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]

    page_number: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class WebformsGetAllWebformsMetaTypedDict(TypedDict):
    total_count: int


class WebformsGetAllWebformsMeta(BaseModel):
    total_count: int


class WebformsGetAllWebformsResponseBodyTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: List[V3WebformsWebformResponseTypedDict]
    meta: WebformsGetAllWebformsMetaTypedDict


class WebformsGetAllWebformsResponseBody(BaseModel):
    r"""The request has succeeded."""

    data: List[V3WebformsWebformResponse]

    meta: WebformsGetAllWebformsMeta


class WebformsGetAllWebformsResponseTypedDict(TypedDict):
    result: WebformsGetAllWebformsResponseBodyTypedDict


class WebformsGetAllWebformsResponse(BaseModel):
    next: Callable[[], Optional[WebformsGetAllWebformsResponse]]

    result: WebformsGetAllWebformsResponseBody



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3WorkflowsEntityOwnerUpdateTypedDict(TypedDict):
    id: NotRequired[str]
    type: NotRequired[str]


class V3WorkflowsEntityOwnerUpdate(BaseModel):
    id: Optional[str] = None

    type: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsPinnedMessageTypedDict(TypedDict):
    r"""Represents a pinned message in the warroom."""

    message: str
    message_sender_id: str
    time: datetime
    time_of_pinning: datetime
    message_id: str
    message_pinned_by: str
    message_sender_name: str


class V3IncidentsPinnedMessage(BaseModel):
    r"""Represents a pinned message in the warroom."""

    message: str

    message_sender_id: str

    time: datetime

    time_of_pinning: datetime

    message_id: str

    message_pinned_by: str

    message_sender_name: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesExtensionsUpdateSlackExtensionRequestTypedDict(TypedDict):
    channel_id: str


class V3ServicesExtensionsUpdateSlackExtensionRequest(BaseModel):
    channel_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_auditlogs_actor import V3AuditLogsActor, V3AuditLogsActorTypedDict
from .v3_auditlogs_team import V3AuditLogsTeam, V3AuditLogsTeamTypedDict
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3AuditLogsAuditLogResponseTypedDict(TypedDict):
    r"""Represents an audit log entry response"""

    id: int
    resource: str
    action: str
    actor: V3AuditLogsActorTypedDict
    r"""Represents an actor (user) in audit logs"""
    client: str
    timestamp: str
    team: V3AuditLogsTeamTypedDict
    r"""Represents a team in audit logs"""


class V3AuditLogsAuditLogResponse(BaseModel):
    r"""Represents an audit log entry response"""

    id: int

    resource: str

    action: str

    actor: V3AuditLogsActor
    r"""Represents an actor (user) in audit logs"""

    client: str

    timestamp: str

    team: V3AuditLogsTeam
    r"""Represents a team in audit logs"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_squadmember import V4SquadsSquadMember, V4SquadsSquadMemberTypedDict
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V4SquadsUpdateSquadRequestTypedDict(TypedDict):
    name: NotRequired[str]
    members: NotRequired[List[V4SquadsSquadMemberTypedDict]]


class V4SquadsUpdateSquadRequest(BaseModel):
    name: Optional[str] = None

    members: Optional[List[V4SquadsSquadMember]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsCommunicationChannelType = Literal[
    "chat_room",
    "video_conference",
    "other",
]


class V3WorkflowsCommunicationChannelTypedDict(TypedDict):
    display_text: str
    link: str
    type: V3WorkflowsCommunicationChannelType


class V3WorkflowsCommunicationChannel(BaseModel):
    display_text: str

    link: str

    type: V3WorkflowsCommunicationChannelType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3IncidentsIncidentExportRequestSortBy = Literal[
    "time",
    "priority",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


CommonV3RBACOwnerType = Literal["team",]
r"""The type of the owner."""


class CommonV3RBACOwnerTypedDict(TypedDict):
    r"""Represents the RBAC owner of an entity."""

    id: str
    r"""The ID of the owner."""
    type: CommonV3RBACOwnerType
    r"""The type of the owner."""


class CommonV3RBACOwner(BaseModel):
    r"""Represents the RBAC owner of an entity."""

    id: str
    r"""The ID of the owner."""

    type: CommonV3RBACOwnerType
    r"""The type of the owner."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from pydantic import model_serializer
from squadcast_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3IncidentsIncidentEventResponseTagsTypedDict(TypedDict):
    pass


class V3IncidentsIncidentEventResponseTags(BaseModel):
    pass


class DeduplicationReasonTypedDict(TypedDict):
    matched_event_id: str
    evaluated_expression: str
    time_window: int


class DeduplicationReason(BaseModel):
    matched_event_id: str

    evaluated_expression: str

    time_window: int


class PayloadTypedDict(TypedDict):
    pass


class Payload(BaseModel):
    pass


class V3IncidentsIncidentEventResponseTypedDict(TypedDict):
    r"""Represents an incident event."""

    incident_id: str
    alert_source_id: str
    message: str
    description: str
    time_of_creation: datetime
    tags: Nullable[V3IncidentsIncidentEventResponseTagsTypedDict]
    payload: PayloadTypedDict
    deduplication_reason: NotRequired[DeduplicationReasonTypedDict]


class V3IncidentsIncidentEventResponse(BaseModel):
    r"""Represents an incident event."""

    incident_id: str

    alert_source_id: str

    message: str

    description: str

    time_of_creation: datetime

    tags: Nullable[V3IncidentsIncidentEventResponseTags]

    payload: Payload

    deduplication_reason: Optional[DeduplicationReason] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["deduplication_reason"]
        nullable_fields = ["tags"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V4StatusPagesIssuesCreateIssueResponseTypedDict(TypedDict):
    id: int
    component_i_ds: List[int]
    title: str


class V4StatusPagesIssuesCreateIssueResponse(BaseModel):
    id: int

    component_i_ds: Annotated[List[int], pydantic.Field(alias="componentIDs")]

    title: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class OverlayDeleteDedupKeyOverlayRequestTypedDict(TypedDict):
    service_id: str
    alert_source: str


class OverlayDeleteDedupKeyOverlayRequest(BaseModel):
    service_id: Annotated[
        str,
        pydantic.Field(alias="serviceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    alert_source: Annotated[
        str,
        pydantic.Field(alias="alertSource"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class OverlayDeleteDedupKeyOverlayResponseBodyTypedDict(TypedDict):
    r"""There is no content to send for this request, but the headers may be useful."""


class OverlayDeleteDedupKeyOverlayResponseBody(BaseModel):
    r"""There is no content to send for this request, but the headers may be useful."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3IncidentsTagsDedupDataTypedDict(TypedDict):
    key: NotRequired[str]
    hash: NotRequired[str]


class V3IncidentsTagsDedupData(BaseModel):
    key: Optional[str] = None

    hash: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class SquadsRemoveSquadMemberRequestTypedDict(TypedDict):
    squad_id: str
    member_id: str
    replace_with: str


class SquadsRemoveSquadMemberRequest(BaseModel):
    squad_id: Annotated[
        str,
        pydantic.Field(alias="squadID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    member_id: Annotated[
        str,
        pydantic.Field(alias="memberID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    replace_with: Annotated[
        str,
        pydantic.Field(alias="replaceWith"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from jsonpath import JSONPath
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.statuspages_componentgroups import StatusPagesComponentGroups
from squadcast_sdk.statuspages_components import StatusPagesComponents
from squadcast_sdk.statuspages_issues import StatusPagesIssues
from squadcast_sdk.statuspages_maintenances_1 import StatusPagesMaintenances1
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class StatusPagesSDK1(BaseSDK):
    components: StatusPagesComponents
    component_groups: StatusPagesComponentGroups
    issues: StatusPagesIssues
    maintenances: StatusPagesMaintenances1

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.components = StatusPagesComponents(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.component_groups = StatusPagesComponentGroups(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.issues = StatusPagesIssues(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.maintenances = StatusPagesMaintenances1(
            self.sdk_configuration, parent_ref=self.parent_ref
        )

    def list(
        self,
        *,
        page_size: int,
        page_number: int,
        filters_is_public: str,
        team_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.StatusPagesListStatusPagesResponse]:
        r"""List Status Pages

        :param page_size:
        :param page_number:
        :param filters_is_public:
        :param team_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesListStatusPagesRequest(
            page_size=page_size,
            page_number=page_number,
            filters_is_public=filters_is_public,
            team_id=team_id,
        )

        req = self._build_request(
            method="GET",
            path="/v4/statuspages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_listStatusPages",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.StatusPagesListStatusPagesResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size
            if len(results[0]) < limit:
                return None

            return self.list(
                page_size=page_size,
                page_number=next_page,
                filters_is_public=filters_is_public,
                team_id=team_id,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.StatusPagesListStatusPagesResponse(
                result=unmarshal_json_response(
                    models.V4StatusPagesListStatusPagesResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        page_size: int,
        page_number: int,
        filters_is_public: str,
        team_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.StatusPagesListStatusPagesResponse]:
        r"""List Status Pages

        :param page_size:
        :param page_number:
        :param filters_is_public:
        :param team_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesListStatusPagesRequest(
            page_size=page_size,
            page_number=page_number,
            filters_is_public=filters_is_public,
            team_id=team_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/statuspages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_listStatusPages",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.StatusPagesListStatusPagesResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page_number
            next_page = page + 1

            if not http_res.text:
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size
            if len(results[0]) < limit:
                return None

            return self.list(
                page_size=page_size,
                page_number=next_page,
                filters_is_public=filters_is_public,
                team_id=team_id,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.StatusPagesListStatusPagesResponse(
                result=unmarshal_json_response(
                    models.V4StatusPagesListStatusPagesResponse, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: str,
        domain_name: str,
        logo_url: str,
        timezone: str,
        team_id: str,
        contact_email: str,
        owner_type: models.V4StatusPagesCreateStatusPageRequestOwnerType,
        owner_id: str,
        description: Optional[str] = None,
        is_public: Optional[bool] = None,
        custom_domain_name: Optional[str] = None,
        theme_color: Optional[
            Union[
                models.V4StatusPagesNewStatusPageThemeColor,
                models.V4StatusPagesNewStatusPageThemeColorTypedDict,
            ]
        ] = None,
        components: Optional[
            Union[
                List[models.V4StatusPagesNewStatusPageComponentList],
                List[models.V4StatusPagesNewStatusPageComponentListTypedDict],
            ]
        ] = None,
        allow_webhook_subscription: Optional[bool] = None,
        allow_components_subscription: Optional[bool] = None,
        allow_maintenance_subscription: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesCreateStatusPageResponse:
        r"""Create Status Page

        :param name:
        :param domain_name:
        :param logo_url:
        :param timezone:
        :param team_id:
        :param contact_email:
        :param owner_type:
        :param owner_id:
        :param description:
        :param is_public:
        :param custom_domain_name:
        :param theme_color:
        :param components:
        :param allow_webhook_subscription:
        :param allow_components_subscription:
        :param allow_maintenance_subscription:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V4StatusPagesCreateStatusPageRequest(
            name=name,
            description=description,
            is_public=is_public,
            domain_name=domain_name,
            custom_domain_name=custom_domain_name,
            logo_url=logo_url,
            timezone=timezone,
            team_id=team_id,
            theme_color=utils.get_pydantic_model(
                theme_color, Optional[models.V4StatusPagesNewStatusPageThemeColor]
            ),
            components=utils.get_pydantic_model(
                components,
                Optional[List[models.V4StatusPagesNewStatusPageComponentList]],
            ),
            contact_email=contact_email,
            allow_webhook_subscription=allow_webhook_subscription,
            allow_components_subscription=allow_components_subscription,
            allow_maintenance_subscription=allow_maintenance_subscription,
            owner_type=owner_type,
            owner_id=owner_id,
        )

        req = self._build_request(
            method="POST",
            path="/v4/statuspages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.V4StatusPagesCreateStatusPageRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_createStatusPage",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesCreateStatusPageResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        domain_name: str,
        logo_url: str,
        timezone: str,
        team_id: str,
        contact_email: str,
        owner_type: models.V4StatusPagesCreateStatusPageRequestOwnerType,
        owner_id: str,
        description: Optional[str] = None,
        is_public: Optional[bool] = None,
        custom_domain_name: Optional[str] = None,
        theme_color: Optional[
            Union[
                models.V4StatusPagesNewStatusPageThemeColor,
                models.V4StatusPagesNewStatusPageThemeColorTypedDict,
            ]
        ] = None,
        components: Optional[
            Union[
                List[models.V4StatusPagesNewStatusPageComponentList],
                List[models.V4StatusPagesNewStatusPageComponentListTypedDict],
            ]
        ] = None,
        allow_webhook_subscription: Optional[bool] = None,
        allow_components_subscription: Optional[bool] = None,
        allow_maintenance_subscription: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesCreateStatusPageResponse:
        r"""Create Status Page

        :param name:
        :param domain_name:
        :param logo_url:
        :param timezone:
        :param team_id:
        :param contact_email:
        :param owner_type:
        :param owner_id:
        :param description:
        :param is_public:
        :param custom_domain_name:
        :param theme_color:
        :param components:
        :param allow_webhook_subscription:
        :param allow_components_subscription:
        :param allow_maintenance_subscription:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V4StatusPagesCreateStatusPageRequest(
            name=name,
            description=description,
            is_public=is_public,
            domain_name=domain_name,
            custom_domain_name=custom_domain_name,
            logo_url=logo_url,
            timezone=timezone,
            team_id=team_id,
            theme_color=utils.get_pydantic_model(
                theme_color, Optional[models.V4StatusPagesNewStatusPageThemeColor]
            ),
            components=utils.get_pydantic_model(
                components,
                Optional[List[models.V4StatusPagesNewStatusPageComponentList]],
            ),
            contact_email=contact_email,
            allow_webhook_subscription=allow_webhook_subscription,
            allow_components_subscription=allow_components_subscription,
            allow_maintenance_subscription=allow_maintenance_subscription,
            owner_type=owner_type,
            owner_id=owner_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/v4/statuspages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.V4StatusPagesCreateStatusPageRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_createStatusPage",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesCreateStatusPageResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def delete_by_id(
        self,
        *,
        statuspage_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesDeleteStatusPageByIDResponse:
        r"""Delete Status Page By ID

        :param statuspage_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesDeleteStatusPageByIDRequest(
            statuspage_id=statuspage_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v4/statuspages/{statuspageID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_deleteStatusPageById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesDeleteStatusPageByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def delete_by_id_async(
        self,
        *,
        statuspage_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesDeleteStatusPageByIDResponse:
        r"""Delete Status Page By ID

        :param statuspage_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesDeleteStatusPageByIDRequest(
            statuspage_id=statuspage_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v4/statuspages/{statuspageID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_deleteStatusPageById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesDeleteStatusPageByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        statuspage_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesGetStatusPageByIDResponse:
        r"""Get Status Page By ID

        :param statuspage_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesGetStatusPageByIDRequest(
            statuspage_id=statuspage_id,
        )

        req = self._build_request(
            method="GET",
            path="/v4/statuspages/{statuspageID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_getStatusPageById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesGetStatusPageByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        statuspage_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesGetStatusPageByIDResponse:
        r"""Get Status Page By ID

        :param statuspage_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesGetStatusPageByIDRequest(
            statuspage_id=statuspage_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/statuspages/{statuspageID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_getStatusPageById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesGetStatusPageByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update(
        self,
        *,
        statuspage_id: str,
        name: str,
        is_public: bool,
        domain_name: str,
        team_id: str,
        theme_color: Union[
            models.V4StatusPagesUpdateStatusPageByIDRequestThemeColor,
            models.V4StatusPagesUpdateStatusPageByIDRequestThemeColorTypedDict,
        ],
        contact_email: str,
        owner_type: str,
        owner_id: str,
        description: Optional[str] = None,
        custom_domain_name: Optional[str] = None,
        logo_url: Optional[str] = None,
        timezone: Optional[str] = None,
        allow_components_subscription: Optional[bool] = None,
        allow_maintenance_subscription: Optional[bool] = None,
        allow_webhook_subscription: Optional[bool] = None,
        components: Optional[
            Union[
                List[models.V4StatusPagesUpdateStatusPageByIDRequestComponent2],
                List[
                    models.V4StatusPagesUpdateStatusPageByIDRequestComponent2TypedDict
                ],
            ]
        ] = None,
        is_custom_domain_enabled: Optional[bool] = None,
        hide_from_search_engines: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesUpdateStatusPageByIDResponse:
        r"""Update Status Page By ID

        :param statuspage_id:
        :param name:
        :param is_public:
        :param domain_name:
        :param team_id:
        :param theme_color:
        :param contact_email:
        :param owner_type:
        :param owner_id:
        :param description:
        :param custom_domain_name:
        :param logo_url:
        :param timezone:
        :param allow_components_subscription:
        :param allow_maintenance_subscription:
        :param allow_webhook_subscription:
        :param components:
        :param is_custom_domain_enabled:
        :param hide_from_search_engines:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesUpdateStatusPageByIDRequest(
            statuspage_id=statuspage_id,
            v4_status_pages_update_status_page_by_id_request=models.V4StatusPagesUpdateStatusPageByIDRequest(
                name=name,
                description=description,
                is_public=is_public,
                domain_name=domain_name,
                custom_domain_name=custom_domain_name,
                logo_url=logo_url,
                timezone=timezone,
                team_id=team_id,
                theme_color=utils.get_pydantic_model(
                    theme_color,
                    models.V4StatusPagesUpdateStatusPageByIDRequestThemeColor,
                ),
                allow_components_subscription=allow_components_subscription,
                allow_maintenance_subscription=allow_maintenance_subscription,
                allow_webhook_subscription=allow_webhook_subscription,
                components=utils.get_pydantic_model(
                    components,
                    Optional[
                        List[models.V4StatusPagesUpdateStatusPageByIDRequestComponent2]
                    ],
                ),
                contact_email=contact_email,
                is_custom_domain_enabled=is_custom_domain_enabled,
                hide_from_search_engines=hide_from_search_engines,
                owner_type=owner_type,
                owner_id=owner_id,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v4/statuspages/{statuspageID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_status_pages_update_status_page_by_id_request,
                False,
                False,
                "json",
                models.V4StatusPagesUpdateStatusPageByIDRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_updateStatusPageById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesUpdateStatusPageByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        statuspage_id: str,
        name: str,
        is_public: bool,
        domain_name: str,
        team_id: str,
        theme_color: Union[
            models.V4StatusPagesUpdateStatusPageByIDRequestThemeColor,
            models.V4StatusPagesUpdateStatusPageByIDRequestThemeColorTypedDict,
        ],
        contact_email: str,
        owner_type: str,
        owner_id: str,
        description: Optional[str] = None,
        custom_domain_name: Optional[str] = None,
        logo_url: Optional[str] = None,
        timezone: Optional[str] = None,
        allow_components_subscription: Optional[bool] = None,
        allow_maintenance_subscription: Optional[bool] = None,
        allow_webhook_subscription: Optional[bool] = None,
        components: Optional[
            Union[
                List[models.V4StatusPagesUpdateStatusPageByIDRequestComponent2],
                List[
                    models.V4StatusPagesUpdateStatusPageByIDRequestComponent2TypedDict
                ],
            ]
        ] = None,
        is_custom_domain_enabled: Optional[bool] = None,
        hide_from_search_engines: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4StatusPagesUpdateStatusPageByIDResponse:
        r"""Update Status Page By ID

        :param statuspage_id:
        :param name:
        :param is_public:
        :param domain_name:
        :param team_id:
        :param theme_color:
        :param contact_email:
        :param owner_type:
        :param owner_id:
        :param description:
        :param custom_domain_name:
        :param logo_url:
        :param timezone:
        :param allow_components_subscription:
        :param allow_maintenance_subscription:
        :param allow_webhook_subscription:
        :param components:
        :param is_custom_domain_enabled:
        :param hide_from_search_engines:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesUpdateStatusPageByIDRequest(
            statuspage_id=statuspage_id,
            v4_status_pages_update_status_page_by_id_request=models.V4StatusPagesUpdateStatusPageByIDRequest(
                name=name,
                description=description,
                is_public=is_public,
                domain_name=domain_name,
                custom_domain_name=custom_domain_name,
                logo_url=logo_url,
                timezone=timezone,
                team_id=team_id,
                theme_color=utils.get_pydantic_model(
                    theme_color,
                    models.V4StatusPagesUpdateStatusPageByIDRequestThemeColor,
                ),
                allow_components_subscription=allow_components_subscription,
                allow_maintenance_subscription=allow_maintenance_subscription,
                allow_webhook_subscription=allow_webhook_subscription,
                components=utils.get_pydantic_model(
                    components,
                    Optional[
                        List[models.V4StatusPagesUpdateStatusPageByIDRequestComponent2]
                    ],
                ),
                contact_email=contact_email,
                is_custom_domain_enabled=is_custom_domain_enabled,
                hide_from_search_engines=hide_from_search_engines,
                owner_type=owner_type,
                owner_id=owner_id,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v4/statuspages/{statuspageID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_status_pages_update_status_page_by_id_request,
                False,
                False,
                "json",
                models.V4StatusPagesUpdateStatusPageByIDRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_updateStatusPageById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesUpdateStatusPageByIDResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def list_statuses(
        self,
        *,
        statuspage_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V4StatusPagesStatusPageStatusesResponse]:
        r"""List Status Page Statuses

        :param statuspage_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesListStatusPageStatusesRequest(
            statuspage_id=statuspage_id,
        )

        req = self._build_request(
            method="GET",
            path="/v4/statuspages/{statuspageID}/status",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_listStatusPageStatuses",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesListStatusPageStatusesResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_statuses_async(
        self,
        *,
        statuspage_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.V4StatusPagesStatusPageStatusesResponse]:
        r"""List Status Page Statuses

        :param statuspage_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.StatusPagesListStatusPageStatusesRequest(
            statuspage_id=statuspage_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/statuspages/{statuspageID}/status",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="StatusPages_listStatusPageStatuses",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.StatusPagesListStatusPageStatusesResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res
            )
            raise errors.BadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedErrorData, http_res
            )
            raise errors.UnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredErrorData, http_res
            )
            raise errors.PaymentRequiredError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ForbiddenErrorData, http_res)
            raise errors.ForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.NotFoundErrorData, http_res)
            raise errors.NotFoundError(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.ConflictErrorData, http_res)
            raise errors.ConflictError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res
            )
            raise errors.UnprocessableEntityError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res
            )
            raise errors.InternalServerError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayErrorData, http_res
            )
            raise errors.BadGatewayError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res
            )
            raise errors.ServiceUnavailableError(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(
                errors.GatewayTimeoutErrorData, http_res
            )
            raise errors.GatewayTimeoutError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_incidentpriority import V3IncidentsIncidentPriority
from .v3_incidents_serviceownerfilter import (
    V3IncidentsServiceOwnerFilter,
    V3IncidentsServiceOwnerFilterTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V3IncidentsExportIncidentsFilterTypedDict(TypedDict):
    r"""Filter criteria for incidents in an export."""

    services: List[str]
    sources: List[str]
    service_owner: V3IncidentsServiceOwnerFilterTypedDict
    r"""Filter criteria for service owner in an export."""
    assigned_to: List[str]
    assigned_to_user_i_ds_and_their_squads: List[str]
    statuses: List[str]
    priority: List[V3IncidentsIncidentPriority]
    tags: List[str]
    notes: str


class V3IncidentsExportIncidentsFilter(BaseModel):
    r"""Filter criteria for incidents in an export."""

    services: List[str]

    sources: List[str]

    service_owner: Annotated[
        V3IncidentsServiceOwnerFilter, pydantic.Field(alias="serviceOwner")
    ]
    r"""Filter criteria for service owner in an export."""

    assigned_to: List[str]

    assigned_to_user_i_ds_and_their_squads: Annotated[
        List[str], pydantic.Field(alias="assignedToUserIDsAndTheirSquads")
    ]

    statuses: List[str]

    priority: List[V3IncidentsIncidentPriority]

    tags: List[str]

    notes: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_createrulesetrequest import (
    V3GlobalEventRulesCreateRulesetRequest,
    V3GlobalEventRulesCreateRulesetRequestTypedDict,
)
from .v3_globaleventrules_rulesetresponse import (
    V3GlobalEventRulesRulesetResponse,
    V3GlobalEventRulesRulesetResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class GlobalEventRulesCreateRulesetRequestTypedDict(TypedDict):
    ger_id: int
    v3_global_event_rules_create_ruleset_request: (
        V3GlobalEventRulesCreateRulesetRequestTypedDict
    )


class GlobalEventRulesCreateRulesetRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    v3_global_event_rules_create_ruleset_request: Annotated[
        V3GlobalEventRulesCreateRulesetRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class GlobalEventRulesCreateRulesetResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3GlobalEventRulesRulesetResponseTypedDict


class GlobalEventRulesCreateRulesetResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3GlobalEventRulesRulesetResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_workflows_communicationchannel import (
    V3WorkflowsCommunicationChannel,
    V3WorkflowsCommunicationChannelTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Literal
from typing_extensions import TypedDict


V3WorkflowsSqAddCommunicationChannelName = Literal["sq_add_communication_channel",]


class V3WorkflowsSqAddCommunicationChannelDataTypedDict(TypedDict):
    channels: List[V3WorkflowsCommunicationChannelTypedDict]


class V3WorkflowsSqAddCommunicationChannelData(BaseModel):
    channels: List[V3WorkflowsCommunicationChannel]


class V3WorkflowsSqAddCommunicationChannelTypedDict(TypedDict):
    name: V3WorkflowsSqAddCommunicationChannelName
    data: V3WorkflowsSqAddCommunicationChannelDataTypedDict


class V3WorkflowsSqAddCommunicationChannel(BaseModel):
    name: V3WorkflowsSqAddCommunicationChannelName

    data: V3WorkflowsSqAddCommunicationChannelData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3WebformsRecaptchaSecretsTypedDict(TypedDict):
    r"""reCAPTCHA credentials to be validated"""

    site_key: str
    r"""reCAPTCHA site key"""
    secret: str
    r"""reCAPTCHA secret key"""


class V3WebformsRecaptchaSecrets(BaseModel):
    r"""reCAPTCHA credentials to be validated"""

    site_key: str
    r"""reCAPTCHA site key"""

    secret: str
    r"""reCAPTCHA secret key"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_extensions_msteams_connectedteams import (
    V3ExtensionsMSTeamsConnectedTeams,
    V3ExtensionsMSTeamsConnectedTeamsTypedDict,
)
from .v3_extensions_msteams_incidentactionalertstate import (
    V3ExtensionsMSTeamsIncidentActionAlertState,
    V3ExtensionsMSTeamsIncidentActionAlertStateTypedDict,
)
from .v3_extensions_msteams_triggers import (
    V3ExtensionsMSTeamsTriggers,
    V3ExtensionsMSTeamsTriggersTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ExtensionsMSTeamsCreateOrUpdateMSTeamsConfigRequestTypedDict(TypedDict):
    r"""The request body for creating or updating an MS Teams extension configuration."""

    default_conversation_name: str
    r"""The display name for the default conversation/channel."""
    default_conversation_id: str
    r"""The unique identifier for the default MS Teams conversation/channel."""
    is_active: bool
    r"""A master switch to enable or disable the entire integration."""
    is_default_active: bool
    r"""Determines if notifications should be sent to the default channel."""
    is_custom_channels_active: bool
    r"""Determines if notifications should be sent to custom-configured channels."""
    triggers: V3ExtensionsMSTeamsTriggersTypedDict
    r"""Configuration for which alerts are sent to the MS Teams channel."""
    tenant_id: str
    r"""The Azure AD Tenant ID of the organization that owns this extension."""
    from_id: str
    r"""The Azure AD Object ID of the user who created this extension."""
    connected_teams: List[V3ExtensionsMSTeamsConnectedTeamsTypedDict]
    r"""A list of all MS Teams (teams) connected to this organization."""
    custom_incident_alert_state: NotRequired[
        V3ExtensionsMSTeamsIncidentActionAlertStateTypedDict
    ]
    r"""A user-friendly way to configure which incident action alerts are active. This is translated by the backend into the 'triggers.custom' array."""
    id: NotRequired[str]
    r"""The MongoDB ObjectID of the extension document. Should be included for updates."""
    organization_id: NotRequired[str]
    r"""The MongoDB ObjectID of the organization this extension belongs to."""


class V3ExtensionsMSTeamsCreateOrUpdateMSTeamsConfigRequest(BaseModel):
    r"""The request body for creating or updating an MS Teams extension configuration."""

    default_conversation_name: str
    r"""The display name for the default conversation/channel."""

    default_conversation_id: str
    r"""The unique identifier for the default MS Teams conversation/channel."""

    is_active: bool
    r"""A master switch to enable or disable the entire integration."""

    is_default_active: bool
    r"""Determines if notifications should be sent to the default channel."""

    is_custom_channels_active: bool
    r"""Determines if notifications should be sent to custom-configured channels."""

    triggers: V3ExtensionsMSTeamsTriggers
    r"""Configuration for which alerts are sent to the MS Teams channel."""

    tenant_id: str
    r"""The Azure AD Tenant ID of the organization that owns this extension."""

    from_id: str
    r"""The Azure AD Object ID of the user who created this extension."""

    connected_teams: List[V3ExtensionsMSTeamsConnectedTeams]
    r"""A list of all MS Teams (teams) connected to this organization."""

    custom_incident_alert_state: Optional[
        V3ExtensionsMSTeamsIncidentActionAlertState
    ] = None
    r"""A user-friendly way to configure which incident action alerts are active. This is translated by the backend into the 'triggers.custom' array."""

    id: Optional[str] = None
    r"""The MongoDB ObjectID of the extension document. Should be included for updates."""

    organization_id: Optional[str] = None
    r"""The MongoDB ObjectID of the organization this extension belongs to."""
